package org.tk.sqle;

import java.io.File;
import java.io.IOException;

import org.tk.sqle.model.Term;
import org.tk.sqle.view.ApplicationView;

/**
 * Programmsteuerung.
 * 
 * @author Thiemo Kahn
 */
public final class Application {

	/** Programmverzeichnis ohne / am Ende */
	private static String root = null;
	/** Trennzeichen fuer Dateipfade */
	private static final String fileSep = System.getProperty("file.separator");
	/** Java Programmverzeichnis */
	private static final String jhome = System.getProperty("java.home");
	/** Zeichen fuer Zeilenumbruch */
	public static final String lineSep = System.getProperty("line.separator");
	/** Trennzeichen fuer Klassenpfade */
	private static final String pathSep = System.getProperty("path.separator");
	/** Klassenpfade */
	private static final String cp = System.getProperty("java.class.path");

	/**
	 * Ermittelt Programmverzeichnis. <br>
	 * 
	 * @return Programmverzeichnis ohne / am Ende.
	 */
	public static String getRoot() {
		if (root == null)
			loadRoot();
		return root;
	}

	/**
	 * Start der Anwendung.<br>
	 * Liest Programmverzeichnis und setzt die Klassenpfade fuer die
	 * Bibliotheken und Treiber. Dazu werden die Verzeichnisse lib und drivers
	 * nach JAR-Dateien durchsucht und auf diese ebenfalls Klassenpfade gesetzt,
	 * und mit diesen das Hauptfenster aufgerufen.<br>
	 * 
	 * @param args
	 *            Komandozeilenargumente werden ignoriert.
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		String[] cp = createClassPath();
		//ApplicationView.main(null); // Zum Debuggen das Fenster direkt aufrufen.
									// Classpath muss gesetzt sein.
		System.out.println("Programmaufruf: " + cp[0] + " " + cp[1] + " " + cp[2] + " " + cp[3]
				+ " " + cp[4]);
		Runtime.getRuntime().exec(cp);
	}

	/**
	 * Laedt Programmverzeichnis.
	 */
	private static void loadRoot() {
		root = null;
		String[] paths = cp.split(pathSep);
		if (paths != null && paths.length > 0) {
			File f = new File(paths[0]);
			String p = f.getAbsolutePath();
			int lio = p.lastIndexOf(fileSep);
			if (lio == -1)
				lio = p.length();
			root = p.substring(0, lio);
			if (root.endsWith("bin"))
				root = root.substring(0, root.length() - 4);
		}
	}

	/**
	 * Baut Klassenpfad fuer den Programmaufruf zusammen.<br>
	 * Hier werden die nachtraeglich zugefuegten JDBC-Treiber gefunden und dem
	 * Klassenpfad hinzugefuegt.
	 * 
	 * @return Um neue JDBC-Treiber ergaenzter Klassenpfad
	 */
	private static String[] createClassPath() {
		String[] start = new String[5];
		String cp = "";
		File dir = new File(getRoot() + fileSep + "lib");
		File[] fs = dir.listFiles();
		for (int j = 0; j < fs.length; j++) {
			if (!fs[j].isDirectory()
					&& (fs[j].getName().endsWith(".jar") || fs[j].getName().endsWith(".zip"))) {
				cp = cp + pathSep + fs[j].getAbsolutePath();
			}
		}
		dir = new File(getRoot() + fileSep + "driver");
		fs = dir.listFiles();
		for (int i = 0; i < fs.length; i++) {
			if (!fs[i].isDirectory()
					&& (fs[i].getName().endsWith(".jar") || fs[i].getName().endsWith(".zip"))) {
				cp = cp + pathSep + fs[i].getAbsolutePath();
			}
		}
		start[0] = jhome + fileSep + "bin" + fileSep + "java";
		start[1] = "-Xmx" + Term.getTerm("XMX");
		start[2] = "-cp";
		start[3] = cp;
		start[4] = "org.tk.sqle.view.ApplicationView";
		return start;
	}
}
