package org.tk.sqle.model;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.tk.sqle.Application;

/**
 * Erzeugen der CSV-Dateien.<br>
 * @author Thiemo Kahn
 */
public final class CSV {

	/**
	 * Exportiert bergebenen Text in eines CSV-Datei.<br>
	 * Es werden nur die Suchergebnisse exportiert. Sie beginnen mit #. Texttrenner ist das Semikolon.
	 * @param text zu speichernde Ausgabe
	 * @param file Datei
	 * @return Meldung
	 */
	public static String exportCsv(String text, File file) throws ActionException {
		try {
			if (!file.getName().toLowerCase().endsWith("csv"))
				file = new File(file.getAbsoluteFile() + ".csv");
			BufferedWriter fw = new BufferedWriter(new FileWriter(file));
			String[] t = text.split(Application.lineSep);
			String[] tn = text.split("\n");
			if (t.length < tn.length)
				t = tn;
			int[] colWidths = null;
			for (int i = 0; i < t.length; i++) {
				if (t[i].startsWith("  --") && i > 0) {
					colWidths = ascertainColWidths(t[i - 1]);
				} else if (t[i].startsWith("#")) {
					t[i] = t[i].substring(2, t[i].length());
					if (colWidths == null) {
						String h = t[i].replaceAll(" +", ";");
						fw.write(h.substring(0, h.length() - 1));
					} else {
						fw.write(createDataLine(t[i], colWidths));
					}
					fw.newLine();
				} else if (t[i].trim().length() == 0) {
					fw.newLine();
				}
			}
			fw.flush();
			fw.close();
		} catch (IOException e) {
			throw new ActionException(Term.getTerm("MSG_DATEI_WEG") + e.getMessage());
		}
		return Term.getTerm("MSG_DATEI_GESCHRIEBEN") + file.getAbsolutePath();
	}

	/**
	 * Trennt die Spalten einer Datenzeile anhand der Spaltenweiten.<br>
	 * @param line Datenzeile
	 * @param colWidths Spaltenweiten
	 * @return mit Semikolon getrennte Datenzeile
	 */
	private static String createDataLine(String line, int[] colWidths) {
		StringBuffer splitLine = new StringBuffer();
		int starti = 0;
		for (int i = 0; i < colWidths.length; i++) {
			String val = line.substring(starti, starti + colWidths[i]);
			starti += colWidths[i];
			splitLine.append(val.trim());
			if (i < colWidths.length - 1) {
				splitLine.append(";");
			}
		}
		return splitLine.toString();
	}

	/**
	 * Ermittelt die Spaltenweiten incl. des einen Leerzeichens, welches die Spalten trennt.<br>
	 * @param headLine Kopfzeile im Original
	 * @return Array mit Spaltenweiten
	 */
	private static int[] ascertainColWidths(String headLine) {
		String[] semicolonLine = headLine.replaceAll(" +", ";").split(";");
		int noCols = semicolonLine.length;
		int[] colWidths = new int[noCols];
		for (int i = 0; i < noCols; i++) {
			if (i == noCols - 1) {
				colWidths[i] = headLine.length() - headLine.indexOf(semicolonLine[i]);
			} else {
				colWidths[i] = headLine.indexOf(semicolonLine[i + 1]) - headLine.indexOf(semicolonLine[i]);
			}
		}
		return colWidths;
	}

}
