package org.tk.sqle.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/** Schreiben und Lesen von Zeichenketten als Dateien ber Datenstrme.
 * @author Thiemo Kahn
 */
public final class FileStringStream {

	/** Liest aus Datei Zeichenkette.<br>
	 * @param path Dateipfad aus dem zu lesen ist
	 * @return String, UTF 8 codiert.
	 */
	public static String read(String path) throws FileNotFoundException, IOException {
		ObjectInputStream ois = null;
		String s = null;
		try {
			File saveFile = new File(path);
			if (saveFile.exists()) {
				ois = new ObjectInputStream(new FileInputStream(saveFile));
				s = ois.readUTF();
			}
		} finally {
			if (ois != null) {
				ois.close();
			}
		}
		return s;
	}

	/** Speichert Zeichenkette als UTF-8-codiert in Datei.<br>
	 * @param string Zeichenkette 
	 * @param path Datei, in die gespeichert wird.
	 */
	public static void write(String string, String path) throws IOException {
		ObjectOutputStream oos = null;
		try {
			File saveFile = new File(path);
			if (!saveFile.exists()) {
				File saveDir = new File(path.substring(0, path.lastIndexOf('/')));
				if (!saveDir.exists())
					saveDir.mkdir();
				saveFile.createNewFile();
			}
			oos = new ObjectOutputStream(new FileOutputStream(saveFile));
			oos.writeUTF(string);
		} finally {
			if (oos != null) {
				oos.close();
			}
		}
	}

}
