package org.tk.sqle.model;

import java.util.Vector;

/**
 * Direkthilfe fuer SQL-Schluesselworte.
 * 
 * @author Thiemo Kahn
 */
public final class SqlHelp {

	private static String stmtSelected = null;
	private static int caretPos = 0;
	private static int start = 0;
	private static int posSelectEnd = -1;

	/**
	 * Fuegt Schluesselwort in aktuelles Statement.
	 * 
	 * @return Um Schluesselwort erweitertes Statement.
	 * @param word
	 *            SQL-Schluesselwort
	 */
	public static String addSqlKeyWord(String word) {
		if (word != null && stmtSelected != null) {
			StringBuffer sb = new StringBuffer(stmtSelected);
			sb.delete(start, caretPos);
			sb.insert(start, word);
			stmtSelected = sb.toString();
			posSelectEnd = start + word.length();
		} else {
			posSelectEnd = -1;
		}
		return stmtSelected;
	}

	/**
	 * Findet Startwert fuer das aktuelle Schluesselwort.
	 */
	private static void getStart() {
		char[] textc = stmtSelected.toCharArray();
		start = 0;
		for (int i = caretPos - 1; i > -1; i--) {
			if (!Character.isLetter(textc[i])) {
				start = i + 1;
				break;
			}
		}
	}

	/**
	 * Liefert Liste von SQL-Schluesselwuertern.
	 * 
	 * @param stmt
	 *            Statement
	 * @param caretPosition
	 *            Zeigerposition
	 * @return Vektor mit SQL-Schluesselwuertern
	 */
	public static Vector getSqlKeyWords(String stmt, int caretPosition) {
		stmtSelected = stmt;
		caretPos = caretPosition;
		Vector l = null;
		if (stmt.length() > 0 && stmt.length() > caretPos - 1) {
			getStart();
			String w = stmt.substring(start, caretPos);
			if (w.length() > 0) {
				l = Help.getSQLKeyWords(w);
			}
		}
		return l;
	}

	/**
	 * Liefert die Zeigerposition nach dem Einfuegen eines Schluesselwortes.
	 * 
	 * @return Zeigerposition nach Einfuegen des Schluesselwortes
	 */
	public static int getCaretPos() {
		return posSelectEnd;
	}

}