package org.tk.sqle.view;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.print.PrinterException;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.jdom.JDOMException;
import org.tk.sqle.Application;
import org.tk.sqle.model.ActionException;
import org.tk.sqle.model.CSV;
import org.tk.sqle.model.DB;
import org.tk.sqle.model.Help;
import org.tk.sqle.model.Log;
import org.tk.sqle.model.SqlHelp;
import org.tk.sqle.model.Term;
import org.tk.sqle.model.Text;

/**
 * Startet Benutzeroberflaeche und fuehrt Aktionen aus.
 * 
 * @author Thiemo Kahn
 */
public final class ApplicationView {

	static DB db = null;

	static WindowMain windowMain = null;
	static WindowHelp windowHelp = null;
	static WindowSQL windowSql = null;
	static WindowSQLDesc windowSQLDesc = null;
	static WindowFileChooserStatements windowFcStmt = null;
	static WindowFileChooserCsv windowFcCsv = null;

	static Autocommit autocommit = null;
	static ClearResult clearResult = null;
	static ClearStmt clearStmt = null;
	static Commit commit = null;
	static Connect connect = null;
	static ExecuteAllStatements executeAllStatements = null;
	static ExecuteStatement executeStatement = null;
	static ExportCsv exportCsv = null;
	static HelpCon helpCon = null;
	static HelpResult helpResult = null;
	static HelpStmt helpStmt = null;
	static Open open = null;
	static PrintResult printResult = null;
	static PrintStatement printStatement = null;
	static Rollback rollback = null;
	static Save save = null;
	static ShowDescription showDescription = null;
	static SQLClose closeSql = null;
	static SQLSelect selectSql = null;
	static SQLShow showSql = null;
	static SQLCloseOnFocus sqlCloseOnFocus = null;
	static SQLLostFocus sqlLostFocus = null;
	static CloseWindow closeWindow = null;

	static final String CONNECT = "connect";
	static final String DISCONNECT = "disconnect";

	/**
	 * Initiatlisiert alle (statischen) Variablen der Klasse. <br>
	 * Reihenfolge: Actions, Haupfenster, Datenbankobjekt, Treiberliste aus
	 * Datenbankobjekt, uebrige Fenster.
	 */
	ApplicationView() {
		closeWindow = new CloseWindow();
		autocommit = new Autocommit();
		clearResult = new ClearResult();
		clearStmt = new ClearStmt();
		commit = new Commit();
		connect = new Connect();
		executeAllStatements = new ExecuteAllStatements();
		executeStatement = new ExecuteStatement();
		exportCsv = new ExportCsv();
		helpCon = new HelpCon();
		helpResult = new HelpResult();
		helpStmt = new HelpStmt();
		open = new Open();
		printResult = new PrintResult();
		printStatement = new PrintStatement();
		rollback = new Rollback();
		save = new Save();
		showDescription = new ShowDescription();
		closeSql = new SQLClose();
		selectSql = new SQLSelect();
		showSql = new SQLShow();
		sqlCloseOnFocus = new SQLCloseOnFocus();
		sqlLostFocus = new SQLLostFocus();
		closeWindow = new CloseWindow();
		windowMain = new WindowMain();
		try {
			db = new DB();
			windowMain.cbUrls.setItems(db.getUrls());
			windowHelp = new WindowHelp(windowMain);
			windowFcStmt = new WindowFileChooserStatements();
			windowFcCsv = new WindowFileChooserCsv();
			windowSql = new WindowSQL(windowMain);
			windowSQLDesc = new WindowSQLDesc(windowSql);
		} catch (ClassNotFoundException e) {
			windowMain.addResultText(Term.getTerm("MSG_NICHT_GEFUNDENER_TREIBER")
					+ Application.lineSep + e.toString());
		} catch (JDOMException e) {
			windowMain.addResultText(Term.getTerm("MSG_NICHT_GELESENER_TREIBER")
					+ Application.lineSep + e.toString());
		} catch (Exception e) {
			windowMain.taResult.setText(e.toString());
		}
	}

	/**
	 * Erzeugt Startfenster und schliesst es, wenn Anwendung gestartet.
	 * 
	 * @param args
	 *            Parameter werden ignoriert
	 */
	public static void main(String[] args) {
		WindowStart ws = new WindowStart();
		new ApplicationView();
		ws.dispose();
		ws = null;
	}

	/**
	 * Beendet beim Schliessen des Fensters eine eventuell offene Verbindung und
	 * speichert letztes Verzeichnis der Statement-Dateien.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class CloseWindow extends WindowAdapter {

		/**
		 * Beendet beim Schliessen des Fensters eine eventuell offene Verbindung
		 * und speichert letztes Verzeichnis der Statement-Dateien.<br>
		 * 
		 * @param e
		 *            klick
		 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
		 */
		public void windowClosing(WindowEvent e) {
			if (windowFcCsv != null && windowFcStmt != null) {
				try {
					windowFcStmt.saveCurrentDir();
					windowFcCsv.saveCurrentDir();
				} catch (IOException e1) {
					Log.error("ApplicationView.WindowClosing: ", e1);
				}
			}
			if (db != null) {
				try {
					db.disconnect();
				} catch (ActionException e1) {
					Log.error("ApplicationView.WindowClosing: ", e1);
				}
			}
			System.exit(0);
		}

	}

	/**
	 * Fuehrt SQL-Statement nach Knopf-Betaetigung aus.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class ExecuteStatement implements ActionListener {

		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			ExecuteRunner ex = new ExecuteRunner();
			Thread t = new Thread(ex);
			t.start();
			windowSql.setVisible(false);
			windowMain.taStmt.requestFocus();
			if (!windowMain.cbCommit.isSelected()) {
				windowMain.bRollback.setEnabled(true);
				windowMain.bCommit.setEnabled(true);
			}
		}
	}

	/**
	 * Fuehrt SQL-Statement aus.<br>
	 * Hilfsklasse fuer ExecuteStatement.
	 * 
	 * @author Thiemo Kahn
	 */
	private class ExecuteRunner implements Runnable {

		/**
		 * Ablauf.
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			StringBuffer b = null;
			try {
				windowMain.setBusy(true);
				b = db.executeStatement(windowMain.taStmt.getCaretPosition(), windowMain.taStmt
						.getText());
				windowMain.addResultText(b.toString());
				b = null;
				System.gc();
			} catch (OutOfMemoryError e0) {
				windowMain.addResultText(Term.getTerm("MSG_OUT_OF_MEMORY"));
			} catch (Throwable e1) {
				windowMain.addResultText(e1.toString());
			}
			windowMain.setBusy(false);
		}

	}

	/**
	 * Fuehrt alle SQL-Statements nach Knopf-Betaetigung aus.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class ExecuteAllStatements implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {

			String[] s = windowMain.taStmt.getText().split(";");
			ExecuteAllRunner ex = new ExecuteAllRunner(s);
			Thread t = new Thread(ex);
			t.start();
			windowSql.setVisible(false);
			windowMain.taStmt.requestFocus();
			if (!windowMain.cbCommit.isSelected()) {
				windowMain.bRollback.setEnabled(true);
				windowMain.bCommit.setEnabled(true);
			}
		}
	}

	/**
	 * Fuehrt alle SQL-Statements aus.<br>
	 * Hilfsklasse fuer ExecuteAllStatements.
	 * 
	 * @author Thiemo Kahn
	 */
	private class ExecuteAllRunner implements Runnable {
		private String[] stmt = null;

		/**
		 * Ablauf.
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			try {
				windowMain.setBusy(true);
				StringBuffer b = null;
				for (int i = 0; i < stmt.length; i++) {
					String st = stmt[i].trim();
					if (st != null && st.length() > 0) {
						b = db.executeStatement(st);
						windowMain.addResultText(b.toString());
					}
				}
			} catch (ActionException e1) {
				windowMain.addResultText(e1.getMessage());
			}
			windowMain.setBusy(false);
		}

		/**
		 * Erzeuger.
		 * 
		 * @param stmt
		 *            SQL-Statements
		 */
		public ExecuteAllRunner(String[] stmt) {
			this.stmt = stmt;
		}

	}

	/**
	 * Rollback.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class Rollback implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			try {
				db.executeStatement("ROLLBACK");
				windowMain.bRollback.setEnabled(false);
				windowMain.bCommit.setEnabled(false);
			} catch (ActionException e1) {
				windowMain.addResultText(e1.toString());
			}
			windowMain.taStmt.requestFocus();
		}
	}

	/**
	 * Commit.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class Commit implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			try {
				db.executeStatement("COMMIT");
				windowMain.bRollback.setEnabled(false);
				windowMain.bCommit.setEnabled(false);
			} catch (ActionException e1) {
				windowMain.addResultText(e1.toString());
			}
			windowMain.taStmt.requestFocus();
		}
	}

	/**
	 * Oeffnet gespreicherte Statement-Dateien.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class Open implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			if (windowFcStmt.showOpenDialog(windowMain) == JFileChooser.APPROVE_OPTION) {
				try {
					windowMain.taStmt.setText(Text.openStatements(windowFcStmt.getSelectedFile()));
					windowMain.lFile.setText(Term.getTerm("TERM_FILE") + ": "
							+ windowFcStmt.getSelectedFile().getName());
					windowMain.lFile.setToolTipText(windowFcStmt.getSelectedFile()
							.getAbsolutePath());
				} catch (ActionException e1) {
					windowMain.addResultText(e1.getMessage());
				}
			}
			windowSql.setVisible(false);
			windowMain.taStmt.requestFocus();
		}
	}

	/**
	 * Speichert aktuelle Statements.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class Save implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			if (windowFcStmt.showSaveDialog(windowMain) == JFileChooser.APPROVE_OPTION) {
				try {
					String s = Text.saveStatements(windowMain.taStmt.getText(), windowFcStmt
							.getSelectedFile());
					windowMain.addResultText(s);
					windowMain.lFile.setText(Term.getTerm("TERM_FILE") + ": "
							+ windowFcStmt.getSelectedFile().getName());
					windowMain.lFile.setToolTipText(windowFcStmt.getSelectedFile()
							.getAbsolutePath());
				} catch (ActionException e1) {
					windowMain.addResultText(e1.getMessage());
				}
			}
			windowSql.setVisible(false);
			windowMain.taStmt.requestFocus();
		}
	}

	/**
	 * Wenn Autocommt ausgewaehlt wurde werden die Rollback- und Commit-Knoepfe
	 * deaktiviert.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class Autocommit implements ItemListener {

		/**
		 * AutoCommit an-/abgewaehlt.
		 * 
		 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
		 */
		public void itemStateChanged(ItemEvent e) {
			if (windowMain.cbCommit.isSelected()) {
				windowMain.bRollback.setEnabled(false);
				windowMain.bCommit.setEnabled(false);
			} else {
				windowMain.bRollback.setEnabled(true);
				windowMain.bCommit.setEnabled(true);
			}

		}

	}

	/**
	 * Hilfe fuer Verbindung.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class HelpCon implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			windowHelp.setVisible(WindowHelp.CON);
		}
	}

	/**
	 * Hilfe fuer Statements.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class HelpStmt implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			windowHelp.setVisible(WindowHelp.STMT);
		}
	}

	/**
	 * Hilfe fuer Ergebnisse.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class HelpResult implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			windowHelp.setVisible(WindowHelp.RES);
		}
	}

	/**
	 * Schliesst bzw. loest Verbindung zu DB nach Knopf-Betaetigung.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class Connect implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			try {
				if (e.getActionCommand().equals(CONNECT)) {
					db.connect((String) windowMain.cbUrls.getSelectedItem(), windowMain.tfUser
							.getText(), windowMain.tfPassword.getPassword(), windowMain.cbCommit
							.isSelected());
					windowMain.bConnect.setStatus(true);
					windowMain.cbUrls.setConnectedItem(db.getConnectedUrl());
					windowMain.taResult.setText("");
					windowMain.cbUrls.setEnabled(false);
					windowMain.tfUser.setEnabled(false);
					windowMain.tfPassword.setEnabled(false);
					windowMain.cbCommit.setEnabled(false);
					windowMain.taStmt.requestFocus();
				} else {
					db.disconnect();
					windowMain.bConnect.setStatus(false);
					windowMain.cbUrls.setEnabled(true);
					windowMain.tfUser.setEnabled(true);
					windowMain.tfPassword.setEnabled(true);
					windowMain.cbCommit.setEnabled(true);
					windowMain.setBusy(false);
				}
			} catch (ActionException e1) {
				windowMain.taResult.setText(e1.getMessage());
			}
			windowSql.setVisible(false);
		}
	}

	/**
	 * SQL-Schluesselwoerter identifizieren.
	 * 
	 * @author Thiemo Kahn
	 */
	private class SQLShow implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getActionCommand().equals("sqlshow")) {
				Vector l = SqlHelp.getSqlKeyWords(windowMain.taStmt.getText().trim(),
						windowMain.taStmt.getCaretPosition());
				if (l != null) {
					Point p = windowMain.taStmt.getCaret().getMagicCaretPosition();
					Point pw = windowMain.getLocation();
					windowSql.setLocation(pw.x + p.x + 11, pw.y + p.y + 132);
					windowSql.setListData(l);
					windowSql.setVisible(true);
				} else {
					windowSql.setVisible(false);
				}
			} else {
				windowSql.setVisible(false);
			}
		}
	}

	/**
	 * Druckt Statements.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class PrintStatement implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			Print p = new Print();
			if (p.setJobOptions()) {
				try {
					p.printText(windowMain.taStmt.getText());
				} catch (PrinterException e1) {
					windowMain.addResultText(e1.getMessage());
				}
			}
			windowSql.setVisible(false);
			windowMain.taStmt.requestFocus();
		}
	}

	/**
	 * Druckt Ergebnisse.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class PrintResult implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			Print p = new Print();
			if (p.setJobOptions()) {
				try {
					p.printText(windowMain.taResult.getText());
				} catch (PrinterException e1) {
					windowMain.addResultText(e1.getMessage());
				}
			}
			windowSql.setVisible(false);
		}
	}

	/**
	 * Exportiert Inhalt des Aufgabebereiches als CSV-Datei.<br>
	 * Das Trennzeichen ist Semikolon.
	 * 
	 * @author Thiemo Kahn
	 */
	private class ExportCsv implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			if (windowFcCsv.showSaveDialog(windowMain) == JFileChooser.APPROVE_OPTION) {
				try {
					String s = CSV.exportCsv(windowMain.taResult.getText(), windowFcCsv
							.getSelectedFile());
					windowMain.addResultText(s);
				} catch (ActionException e1) {
					windowMain.addResultText(e1.getMessage());
				}
			}
		}
	}

	/**
	 * Leert Statementbereich.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class ClearStmt implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			windowSql.setVisible(false);
			windowMain.taStmt.requestFocus();
			windowMain.taStmt.setText(null);
			windowMain.lFile.setText(null);
		}
	}

	/**
	 * Leert Ergebnissbereich.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	private class ClearResult implements ActionListener {
		/**
		 * @param e
		 *            klick
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			windowMain.taStmt.requestFocus();
			windowMain.taResult.setText(null);
		}
	}

	/**
	 * SQL-Schluesselwort uebernehmen und Direkthilfe ausblenden.
	 * 
	 * @author Thiemo Kahn
	 */
	private class SQLSelect extends KeyAdapter {
		boolean ctrl = false;

		/**
		 * @param e
		 *            Eingabetaste gedrueckt
		 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
		 */
		public void keyReleased(KeyEvent e) {
			if (e.getKeyCode() == KeyEvent.VK_ENTER) {
				windowSql.setVisible(false);
				windowMain.taStmt.setText(SqlHelp.addSqlKeyWord((String) windowSql.lWords
						.getSelectedValue()));
				int p = SqlHelp.getCaretPos();
				if (p > -1)
					windowMain.taStmt.setCaretPosition(p);
			}
		}
	}

	/**
	 * Direkthilfe ausblenden.
	 * 
	 * @author Thiemo Kahn
	 */
	private class SQLClose implements ActionListener {
		/**
		 * @param e
		 *            sqlclose
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getActionCommand().equals("sqlclose")) {
				windowSql.setVisible(false);
			}
		}
	}

	/**
	 * Beschreibung zu ausgewaehltem Schluesselwort anzeigen.
	 * 
	 * @author Thiemo Kahn
	 */
	private class ShowDescription implements ListSelectionListener {
		/**
		 * @param e
		 *            Auswahl geaendert
		 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
		 */
		public void valueChanged(ListSelectionEvent e) {
			String key = (String) windowSql.lWords.getSelectedValue();
			String d = Help.getDescriptionForSQLKeyWord(key);
			if (d != null && d.length() > 0) {
				windowSQLDesc.setText(d);
				windowSQLDesc.setLocation(windowSql.getX() + windowSql.getWidth() + 2, windowSql
						.getY());
				windowSQLDesc.setVisible(true);
				// TODO
				// windowSql.setVisible(true);
			} else {
				windowSQLDesc.setVisible(false);
			}
		}
	}

	/**
	 * Close SQL-Help, if focus is gained.
	 * 
	 * @author Thiemo Kahn
	 * 
	 */
	private class SQLCloseOnFocus implements FocusListener {

		/**
		 * Close SQL-Help, if focus is gained.
		 * 
		 * @param e
		 *            klick
		 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
		 */
		public void focusGained(FocusEvent e) {
			windowSql.setVisible(false);
		}

		/**
		 * Nothing happens
		 * 
		 * @param e
		 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
		 */
		public void focusLost(FocusEvent e) {

		}

	}

	/**
	 * Get the focus back to SQL-Help, if its lost by showing Description.
	 * 
	 * @author Thiemo Kahn
	 * 
	 */
	private class SQLLostFocus implements FocusListener {

		/**
		 * Get the focus back to SQL-Help
		 * 
		 * @param e
		 *            klick
		 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
		 */
		public void focusGained(FocusEvent e) {
			windowSql.lWords.requestFocus();
		}

		/**
		 * Nothing happens
		 * 
		 * @param e
		 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
		 */
		public void focusLost(FocusEvent e) {

		}

	}

}
