package org.tk.sqle.view;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.math.BigDecimal;

import org.tk.sqle.Application;

/** Druck.<br>
 * Aufruf:
 * Print p = new Print();
 * if (p.setJobOptions()) {
 * try {
 * 		p.printText(taStmt.getText());
 * 	} catch (PrinterException e1) {
 * 		taResult.setText(taResult.getText() + e1.getMessage());
 * 	}
 * @author Thiemo Kahn
 */
final class Print implements Printable {

	private PrinterJob pJob = null;
	private String[] textLines = null;
	/** x-Koordinate fr Beginn des druckbaren Bereiches */
	private double py = 0;
	/** y-Koordinate fr Beginn des druckbaren Bereiches s*/
	private double px = 0;
	/** Zeilen je Seite */
	private int lpp = 0;
	/** Anzahl Seiten */
	private int nop = 0;
	/** Zeilenhhe */
	private int lh = 0;
	/** Orientierung */
	private int orientation = PageFormat.PORTRAIT;
	/** Schrift */
	private Font font = null;

	/** Erzeugt einen Druck. 
	 */
	Print() {
		super();
		pJob = PrinterJob.getPrinterJob();
		pJob.setPrintable(this);
	}

	/** Fhrt Dialog fr die Druckoptionen aus.
	 * @return Wenn true, dann hat Benutzer ok geklickt und es wurde ein 
	 * Druck-Auftrag erzeugt.
	 */
	boolean setJobOptions() {
		return pJob.printDialog();
	}

	/** Druckt Text aus Text-Komponente. 
	 * Zuvor mu setJobOptions ausgefhrt worden sein,
	 * damit ein Druck-Auftrag vorhanden ist.
	 * @param comp Textkomponente des GUI
	 */
	void printText(String text) throws PrinterException {
		String[] tln = text.split("\n");
		this.textLines = text.split(Application.lineSep);
		if(this.textLines.length < tln.length)
			this.textLines = tln;
		pJob.print();

	}

	/** Wird von PrinterJob zum Drucken verwendet.
	 * @param graphics Grafik
	 * @param pageFormat Seitenformat
	 * @param pageIndex Seitenindex
	 * @return Folgeseite oder nicht
	 * @see java.awt.print.Printable#print(java.awt.Graphics, java.awt.print.PageFormat, int)
	 */
	public int print(Graphics graphics, PageFormat pageFormat, int pageIndex)
		throws PrinterException {
		int r = NO_SUCH_PAGE;
		if (pageIndex == 0) {
			py = pageFormat.getImageableY();
			px = pageFormat.getImageableX();
			compressLine(graphics, pageFormat);
			nop = noOfPages(graphics, pageFormat);
			lpp = linesPerPage(graphics, pageFormat);
			lh = lineHeight(graphics);
		} else {
			graphics.setFont(font);
			pageFormat.setOrientation(orientation);
		}
		int start = pageIndex * lpp;
		int end = start + lpp;
		if (end > textLines.length)
			end = textLines.length;
		int j = 0;
		for (int i = start; i < end; i++) {
			graphics.drawString(textLines[i], (int) px, (int) (py + j * lh + 9));
			j++;
		}
		if (pageIndex < nop)
			r = PAGE_EXISTS;
		return r;
	}

	/** Berechnet Zeilenhhe.
	 * Zuvor sollte ber compressLine die Schriftge festgelegt sein.
	 * @param graphics Grafik
	 * @return Zeilenhhe.
	 */
	private int lineHeight(Graphics graphics) {
		return graphics.getFontMetrics().getHeight();
	}

	/** Berechnet Anzahl der Seiten.
	 * Zuvor sollte ber compressLine die Schriftge festgelegt sein.
	 * @param graphics Grafik
	 * @param pageFormat Seitenformat
	 * @return Anzahl Seiten.
	 */
	private int noOfPages(Graphics graphics, PageFormat pageFormat) {
		int no = 0;
		int lh = lineHeight(graphics);
		int ph = (int) pageFormat.getImageableHeight();
		no =
			new BigDecimal(textLines.length * lh)
				.divide(new BigDecimal(ph), BigDecimal.ROUND_UP)
				.intValue();
		return no;
	}

	/** Berechnet Anzahl der Zeilen je Seite.
	 * Zuvor sollte ber compressLine die Schriftge festgelegt sein.
	 * @param graphics Grafik
	 * @param pageFormat Seitenformat
	 * @return Anzahl Zeilen je Seite.
	 */
	private int linesPerPage(Graphics graphics, PageFormat pageFormat) {
		int no = 0;
		int lh = lineHeight(graphics);
		double ph = pageFormat.getImageableHeight();
		no = new BigDecimal(ph).divide(new BigDecimal(lh), BigDecimal.ROUND_DOWN).intValue();
		return no;
	}
	

	/** Presst Text in Seitenbreite durch schrittweises Verringern der Schriftgre
	 * und, wenn das nicht reicht, durch Wechsel auf Querformat.
	 * @param graphics Grafik
	 * @param pageFormat Seitenformat
	 */
	private void compressLine(Graphics graphics, PageFormat pageFormat) {
		graphics.setFont(Fonts.SMALL_MONO);
		font = Fonts.SMALL_MONO;
		FontMetrics fm = graphics.getFontMetrics();
		int pw = (int) pageFormat.getImageableWidth();
		for (int i = 0; i < textLines.length; i++) {
			int lw = fm.stringWidth(textLines[i]);
			if (lw > pw) {
				if (graphics.getFont().equals(Fonts.SMALL_MONO)) {
					graphics.setFont(Fonts.XSMALL_MONO);
					font = Fonts.XSMALL_MONO;
					fm = graphics.getFontMetrics();
					i--;
				} else if (graphics.getFont().equals(Fonts.XSMALL_MONO)) {
					graphics.setFont(Fonts.XXSMALL_MONO);
					font = Fonts.XXSMALL_MONO;
					fm = graphics.getFontMetrics();
					i--;
				} else if (graphics.getFont().equals(Fonts.XXSMALL_MONO)) {
					pageFormat.setOrientation(PageFormat.LANDSCAPE);
					orientation = PageFormat.LANDSCAPE;
					break;
				}
			}
		}
	}

}
