package org.tk.sqle.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import org.tk.sqle.model.Term;
import org.tk.sqle.Application;

/**
 * Rahmen fuer das Haupt-Dialogfenster mit dem Editor fuer die Statements.<br>
 * 
 * @author Thiemo Kahn
 */
final class WindowMain extends JFrame {

	private static final long serialVersionUID = 6099930795408604557L;
	ButtonExecute bExecute = new ButtonExecute();
	ButtonExecuteAll bExecuteAll = new ButtonExecuteAll();
	ButtonPrint bPrintStmt = new ButtonPrint();
	ButtonPrint bPrintRes = new ButtonPrint();
	ButtonOpen bOpen = new ButtonOpen();
	ButtonSave bSave = new ButtonSave();
	ButtonHelp bHelpCon = new ButtonHelp();
	ButtonHelp bHelpStmt = new ButtonHelp();
	ButtonHelp bHelpResult = new ButtonHelp();
	ButtonClear bClearStmt = new ButtonClear();
	ButtonClear bClearResult = new ButtonClear();
	ButtonCommit bCommit = new ButtonCommit();
	ButtonRollback bRollback = new ButtonRollback();
	ButtonCsv bCsv = new ButtonCsv();
	ButtonConnect bConnect = new ButtonConnect();
	TextAreaStatements taStmt = new TextAreaStatements();
	TextAreaResult taResult = new TextAreaResult();
	ComboBoxUrls cbUrls = new ComboBoxUrls();
	TextFieldUser tfUser = new TextFieldUser();
	PasswordFieldPassword tfPassword = new PasswordFieldPassword();
	CheckBoxAutoCommit cbCommit = new CheckBoxAutoCommit();
	JLabel lFile = new JLabel();

	JPanel pStmt = null;
	JPanel pResult = null;
	JSplitPane panelEdit = null;

	/**
	 * Erzeugt Haupt-Fenster.
	 */
	WindowMain() {
		super("sqleditor 0.9.4");
		setIconImage(getToolkit().getImage(Application.getRoot() + "/gfx/eule32.png"));
		setDimension();
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(ApplicationView.closeWindow);
		BorderLayout lay = new BorderLayout(0, 4);
		getContentPane().setLayout(lay);
		pStmt = createPanelStatement();
		pResult = createPanelResult();
		panelEdit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, pStmt, pResult);
		panelEdit.setDividerLocation(250);
		panelEdit.setOneTouchExpandable(true);
		getContentPane().add(createPanelConnect(), BorderLayout.NORTH);
		getContentPane().add(panelEdit, BorderLayout.CENTER);
		setVisible(true);
	}

	/**
	 * Baut Bereich mit Connection-Elementen zusammen.
	 * 
	 * @return Verbindungsbereich
	 */
	private JPanel createPanelConnect() {
		GridBagLayout lay = new GridBagLayout();
		JPanel panel = new JPanel(lay);
		panel.setBorder(BorderFactory.createLineBorder(Colors.BORDER_COLOR));
		JLabel lUrls = new JLabel(Term.getTerm("TERM_JDBC_URL"));
		JLabel lUser = new JLabel(Term.getTerm("TERM_BENUTZER"));
		JLabel lPassword = new JLabel(Term.getTerm("TERM_PW"));
		JLabel lCommit = new JLabel(Term.getTerm("TERM_AUTO_COMMIT"));
		LabelImpressum lImp = new LabelImpressum();
		bConnect.addActionListener(ApplicationView.connect);
		bHelpCon.addActionListener(ApplicationView.helpCon);
		cbCommit.addItemListener(ApplicationView.autocommit);
		addComponent(lUrls, 0, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(lUser, 1, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(lPassword, 2, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(lCommit, 3, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(lImp, 4, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay,
				GridBagConstraints.NORTH);
		addComponent(bHelpCon, 5, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay,
				GridBagConstraints.NORTHEAST);
		addComponent(cbUrls, 0, 1, 1, 1, 1.0, 0, GridBagConstraints.HORIZONTAL, panel, lay, -1);
		addComponent(tfUser, 1, 1, 1, 1, 0.8, 0, GridBagConstraints.HORIZONTAL, panel, lay, -1);
		addComponent(tfPassword, 2, 1, 1, 1, 0.8, 0, GridBagConstraints.HORIZONTAL, panel, lay, -1);
		addComponent(cbCommit, 3, 1, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bConnect, 4, 1, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		return panel;
	}

	/**
	 * Baut Statementbereich zusammen.
	 * 
	 * @return Statementbereich
	 */
	private JPanel createPanelStatement() {
		GridBagLayout lay = new GridBagLayout();
		JPanel panel = new JPanel(lay);
		panel.setBorder(BorderFactory.createLineBorder(Colors.BORDER_COLOR));
		JLabel lStmt = new JLabel(Term.getTerm("TERM_STATEMENTS"));
		bExecute.addActionListener(ApplicationView.executeStatement);
		bExecuteAll.addActionListener(ApplicationView.executeAllStatements);
		bPrintStmt.addActionListener(ApplicationView.printStatement);
		bPrintStmt.setMnemonic('p');
		taStmt.registerKeyboardAction(ApplicationView.showSql, "sqlshow", KeyStroke.getKeyStroke(
				KeyEvent.VK_SPACE, 2), JComponent.WHEN_FOCUSED);
		bOpen.addActionListener(ApplicationView.open);
		bSave.addActionListener(ApplicationView.save);
		bHelpStmt.addActionListener(ApplicationView.helpStmt);
		bClearStmt.addActionListener(ApplicationView.clearStmt);
		bClearStmt.setMnemonic('l');
		bRollback.addActionListener(ApplicationView.rollback);
		bCommit.addActionListener(ApplicationView.commit);
		lFile.setFont(Fonts.REG);
		addComponent(lStmt, 0, 0, 1, 1, 1.0, 0, GridBagConstraints.HORIZONTAL, panel, lay, -1);
		addComponent(lFile, 1, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bClearStmt, 2, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bOpen, 3, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bSave, 4, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bPrintStmt, 5, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bRollback, 6, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bExecuteAll, 7, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bExecute, 8, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bCommit, 9, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bHelpStmt, 10, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay,
				GridBagConstraints.NORTHEAST);
		addComponent(new JScrollPane(taStmt), 0, 1, 11, 1, 1.0, 1.0, GridBagConstraints.BOTH,
				panel, lay, -1);
		return panel;
	}

	/**
	 * Baut Ausgabebereich zusammen.
	 * 
	 * @return Ausgabebereich
	 */
	private JPanel createPanelResult() {
		GridBagLayout lay = new GridBagLayout();
		JPanel panel = new JPanel(lay);
		panel.setBorder(BorderFactory.createLineBorder(Colors.BORDER_COLOR));
		JLabel lResult = new JLabel(Term.getTerm("TERM_AUSGABEN"));
		bPrintRes.addActionListener(ApplicationView.printResult);
		bPrintRes.setMnemonic('q');
		bHelpResult.addActionListener(ApplicationView.helpResult);
		bClearResult.addActionListener(ApplicationView.clearResult);
		bClearResult.setMnemonic('m');
		bCsv.addActionListener(ApplicationView.exportCsv);
		addComponent(lResult, 0, 0, 1, 1, 1.0, 0, GridBagConstraints.HORIZONTAL, panel, lay, -1);
		addComponent(bClearResult, 1, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bPrintRes, 2, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bCsv, 3, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay, -1);
		addComponent(bHelpResult, 4, 0, 1, 1, 0, 0, GridBagConstraints.NONE, panel, lay,
				GridBagConstraints.NORTHEAST);
		addComponent(new JScrollPane(taResult), 0, 1, 5, 1, 1.0, 1.0, GridBagConstraints.BOTH,
				panel, lay, -1);
		return panel;
	}

	/**
	 * Fuegt Komponente dem Fenster hinzu.
	 * 
	 * @param c
	 *            Komponente
	 * @param gridx
	 *            Spalte
	 * @param gridy
	 *            Zeile
	 * @param gridwidth
	 *            Weite ueber Spalten
	 * @param gridheight
	 *            Hoehe ueber Zeilen
	 * @param weightx
	 *            Anteil Ausdehnung Spalte
	 * @param weighty
	 *            Anteil Ausdehnung Zeile
	 * @param fill
	 *            Richtung der Ausdehnung
	 * @param p
	 *            Panel
	 * @param lay
	 *            Layout-Manager
	 * @param pos
	 *            Position der Komponente im Feld
	 */
	private void addComponent(Component c, int gridx, int gridy, int gridwidth, int gridheight,
			double weightx, double weighty, int fill, JPanel p, GridBagLayout lay, int pos) {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = fill;
		gbc.gridx = gridx;
		gbc.gridy = gridy;
		gbc.gridwidth = gridwidth;
		gbc.gridheight = gridheight;
		gbc.weightx = weightx;
		gbc.weighty = weighty;
		if (pos == -1)
			gbc.anchor = GridBagConstraints.NORTHWEST;
		else
			gbc.anchor = pos;
		gbc.insets = new Insets(6, 2, 2, 2);
		lay.setConstraints(c, gbc);
		p.add(c);
	}

	/**
	 * Legt Groesse und Position des Rahmens fest.
	 */
	private void setDimension() {
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		if (d.width > 800) {
			setSize(800, 600);
		} else {
			setSize(d.width, d.height - 30);
		}
		setLocation(0, 0);
	}

	/**
	 * Knopf zum Leeren eines Textbereichs.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonClear extends JButton {

		private static final long serialVersionUID = 1435694556699973006L;

		/**
		 * Erzeuger.
		 */
		ButtonClear() {
			super();
			setBorderPainted(false);
			setToolTipText(Term.getTerm("DESC_LEEREN"));
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/clear.png"));
		}

	}

	/**
	 * Commit-Knopf.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonCommit extends JButton {

		private static final long serialVersionUID = -2410484689293828237L;

		/**
		 * Erzeuger.
		 */
		ButtonCommit() {
			super();
			setBorderPainted(false);
			setToolTipText(Term.getTerm("DESC_COMMIT"));
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/commit.png"));
			setMnemonic('t');
		}

	}

	/**
	 * Der Knopf, der Verbindung zur DB herstellt und wieder loest.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonConnect extends JButton {

		private static final long serialVersionUID = -4744687705754819646L;
		private boolean connected = false;

		/**
		 * Erzeuger.
		 */
		ButtonConnect() {
			super();
			connected = false;
			setActionCommand(ApplicationView.CONNECT);
			setBorderPainted(false);
			setFont(Fonts.REG);
			setToolTipText(Term.getTerm("DESC_VERBINDEN"));
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/disconnect.png"));
			setMnemonic('c');
		}

		/**
		 * Setzt Verbindungsstatus.
		 * 
		 * @param connected
		 *            Wenn true, dann verbunden
		 */
		void setStatus(boolean connected) {
			this.connected = connected;
			if (connected == true) {
				setToolTipText(Term.getTerm("DESC_ENTBINDEN"));
				setActionCommand(ApplicationView.DISCONNECT);
				setIcon(new ImageIcon(Application.getRoot() + "/gfx/connect.png"));
			} else {
				setToolTipText(Term.getTerm("DESC_VERBINDEN"));
				setActionCommand(ApplicationView.CONNECT);
				setIcon(new ImageIcon(Application.getRoot() + "/gfx/disconnect.png"));
			}
		}

	}

	/**
	 * Commit-Knopf.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonCsv extends JButton {

		private static final long serialVersionUID = 7024627362416517360L;

		/**
		 * Erzeuger.
		 */
		ButtonCsv() {
			super();
			setBorderPainted(false);
			setToolTipText(Term.getTerm("DESC_CSV"));
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/csv.png"));
			setMnemonic('v');
		}

	}

	/**
	 * Der Knopf, der das Statement ausf�hrt.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonExecute extends JButton {

		private static final long serialVersionUID = 1408403323012030632L;

		/**
		 * Erzeuger
		 */
		ButtonExecute() {
			super();
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/run.png"));
			setFont(Fonts.REG);
			setToolTipText(Term.getTerm("DESC_AUSFUEHREN"));
			setBorderPainted(false);
			setMnemonic('e');
		}

	}

	/**
	 * Der Knopf, der alle Statements ausfuehrt.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonExecuteAll extends JButton {

		private static final long serialVersionUID = -3648413387719471634L;

		/**
		 * Erzeuger.
		 */
		ButtonExecuteAll() {
			super();
			setBorderPainted(false);
			setFont(Fonts.REG);
			setToolTipText(Term.getTerm("DESC_ALLE_AUSFUEHREN"));
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/runall.png"));
			setMnemonic('a');
		}

	}

	/**
	 * Hilfe Knopf.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonHelp extends JButton {

		private static final long serialVersionUID = 8125883473485163110L;

		/**
		 * Erzeuger.
		 */
		ButtonHelp() {
			super(Term.getTerm("ABBR_HILFE"));
			setFont(Fonts.BIG_BOLD);
			setToolTipText(Term.getTerm("DESC_HILFE"));
			setForeground(Colors.BORDER_COLOR);
			setBorder(null);
		}

	}

	/**
	 * Oeffnen Knopf.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonOpen extends JButton {

		private static final long serialVersionUID = -3454983104111752641L;

		/**
		 * Erzeuger.
		 */
		ButtonOpen() {
			super();
			setBorderPainted(false);
			setFont(Fonts.REG);
			setToolTipText(Term.getTerm("DESC_OEFFNEN"));
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/open.png"));
			setMnemonic('o');
		}

	}

	/**
	 * Druckknopf fuer Ergebnisse.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonPrint extends JButton {

		private static final long serialVersionUID = -1356684381769662884L;

		/**
		 * Erzeuger.
		 */
		ButtonPrint() {
			super();
			setBorderPainted(false);
			setToolTipText(Term.getTerm("DESC_DRUCKEN"));
			setFont(Fonts.REG);
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/print.png"));

		}

	}

	/**
	 * Rollback-Knopf.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonRollback extends JButton {

		private static final long serialVersionUID = -3907349141924382798L;

		/**
		 * Erzeuger.
		 */
		ButtonRollback() {
			super();
			setBorderPainted(false);
			setToolTipText(Term.getTerm("DESC_ROLLBACK"));
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/rollback.png"));
			setMnemonic('r');
		}

	}

	/**
	 * Speichern Knopf.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ButtonSave extends JButton {

		private static final long serialVersionUID = -5372959531453056305L;

		/**
		 * Erzeuger.
		 */
		ButtonSave() {
			super();
			setBorderPainted(false);
			setFont(Fonts.REG);
			setToolTipText(Term.getTerm("DESC_SPEICHERN"));
			setIcon(new ImageIcon(Application.getRoot() + "/gfx/save.png"));
			setMnemonic('s');
		}

	}

	/**
	 * Checkbox fuer Autocommit.
	 * 
	 * @author Thiemo Kahn
	 */
	final class CheckBoxAutoCommit extends JCheckBox {

		private static final long serialVersionUID = 7733151761903833682L;
		private boolean autoCommit = true;

		/**
		 * Erzeuger.
		 */
		CheckBoxAutoCommit() {
			super();
			autoCommit = true;
			setBorderPainted(true);
			setSelected(false);
			setFont(Fonts.REG);
			setMnemonic('u');
		}

		/**
		 * Setzt Commit-Steuerung.
		 * 
		 * @param autoCommit
		 *            automatisch=true, manuell=false
		 */
		void setAutoCommit(boolean autoCommit) {
			this.autoCommit = autoCommit;
			if (autoCommit == true) {
				setSelected(true);
			} else {
				setSelected(false);
			}
		}

	}

	/**
	 * Auswahlfeld fuer DB-Treiber-URLs.
	 * 
	 * @author Thiemo Kahn
	 */
	final class ComboBoxUrls extends JComboBox {

		private static final long serialVersionUID = -8138853742961635699L;
		private int connectedIndex = -1;
		private List urls = null;

		/**
		 * Erzeugt Auswahlfeld mit den in drivers.xml gefundenen URLs.
		 */
		ComboBoxUrls() {
			super();
			setFont(Fonts.REG);
			setEditable(true);
		}

		/**
		 * Setzt ausgewaehlte URL. Sie wird hervorgehoben.
		 * 
		 * @param connectedUrl
		 *            JDBC-URL
		 */
		void setConnectedItem(String connectedUrl) {
			connectedIndex = -1;
			if (connectedUrl != null) {
				int size = getItemCount();
				for (int i = 0; i < size; i++) {
					if (getItemAt(i).equals(connectedUrl)) {
						connectedIndex = i;
						break;
					}
				}
			}
			setSelectedItem(connectedUrl);
		}

		/**
		 * Setzt Inhalte der Auswahlbox.
		 * 
		 * @param urls
		 *            Liste mit JDBC-URLs
		 */
		void setItems(List urls) {
			this.urls = urls;
			connectedIndex = -1;
			if (urls != null) {
				for (Iterator iter = urls.iterator(); iter.hasNext();) {
					addItem((String) iter.next());
				}
			}
		}

	}

	/**
	 * Impressum.
	 * 
	 * @author Thiemo Kahn
	 */
	final class LabelImpressum extends JLabel {

		private static final long serialVersionUID = -3601388258813690556L;

		/**
		 * Impressum Konstruktor.
		 */
		public LabelImpressum() {
			super("" + (char) 169);
			setToolTipText(Term.getTerm("DESC_IMPRESSUM"));
			setFont(Fonts.BIG_BOLD);
			setForeground(Colors.BORDER_COLOR);
		}

	}

	/**
	 * Passwortfeld.
	 * 
	 * @author Thiemo Kahn
	 */
	final class PasswordFieldPassword extends JPasswordField {

		private static final long serialVersionUID = 6785401650745178761L;

		/**
		 * Erzeuger.
		 */
		PasswordFieldPassword() {
			super(8);
			setFont(Fonts.REG);
		}

	}

	/**
	 * Mehrzeiliges Textfeld fuer die Ausgabe.<br>
	 * Z.B. Ergebnis eines Select-Statements oder SQL-Fehler.
	 * 
	 * @author Thiemo Kahn
	 */
	final class TextAreaResult extends JTextArea {

		private static final long serialVersionUID = -6677272943012775451L;

		/**
		 * Erzeuger.
		 */
		TextAreaResult() {
			super();
			setFont(Fonts.REG_MONO);
		}

	}

	/**
	 * Mehrzeiliges Textfeld fuer die SQL-Statements.<br>
	 * 
	 * @author Thiemo Kahn
	 */
	final class TextAreaStatements extends JTextArea {

		private static final long serialVersionUID = 7730778632633558376L;

		/**
		 * Erzeugt Textfeld.
		 */
		TextAreaStatements() {
			super();
			setLineWrap(true);
			setWrapStyleWord(true);
			setFont(Fonts.REG_MONO);
			addFocusListener(ApplicationView.sqlCloseOnFocus);			
		}

	}

	/**
	 * Eingabefeld fuer Benutzername.
	 * 
	 * @author Thiemo Kahn
	 */
	final class TextFieldUser extends JTextField {

		private static final long serialVersionUID = 3922033882330324612L;

		/**
		 * Erzeuger.
		 */
		TextFieldUser() {
			super(8);
			setFont(Fonts.REG);
		}

	}

	/**
	 * Sperrt Ausfuehren-Knoepfe und zeigt Eieruhr.
	 * 
	 * @param b
	 *            Wenn true, dann beschaeftigt.
	 */
	void setBusy(boolean b) {
		if (b == true) {
			bExecute.setEnabled(false);
			bExecuteAll.setEnabled(false);
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
			taStmt.setCursor(new Cursor(Cursor.WAIT_CURSOR));
			taResult.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		} else {
			bExecute.setEnabled(true);
			bExecuteAll.setEnabled(true);
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			taStmt.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			taResult.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
	}

	/**
	 * Ergaenzt Text in Ausgabe.
	 * 
	 * @param text
	 *            Ausgabetext
	 */
	void addResultText(String text) {
		taResult.append(Application.lineSep + text);
		int c = taResult.getText().length()
				- (text.length() - text.lastIndexOf(Application.lineSep));
		taResult.moveCaretPosition(c);
	}

}
